
import bpy
from .op_cube import BoxCreateOperator
from .op_cornercube import CornerCubeCreateOperator
from .op_cylinder import CylinderCreateOperator
from .op_sphere import SphereCreateOperator
from .op_spherecube import SphereCubeCreateOperator
from .op_pyramid import PyramidCreateOperator
from .op_torus import TorusCreateOperator
from .op_transform import ObjectTransformOperator

from .qblock_prop import QBlockProperties, MESH_PT_QblockPanel
from .qblock.qblock_convert import ConvertToQBlock


bl_info = {
    "name": "QBlocker Free",
    "author": "xz-blender repack and translate",
    "version": (0, 1, 6, 1),
    "description": "",
    "blender": (4, 0, 0),
    "location": "View3D > Add > QBlocker",
    "warning": "",
    "category": "Add Mesh",
    "doc_url": "https://qblockerdocs.readthedocs.io/qblocker_free.html"
}


class QBlockerAddonPreferences(bpy.types.AddonPreferences):
    bl_idname = __package__

    # General settings #
    mouse_enum: bpy.props.EnumProperty(
        name="鼠标选择模式",
        items=(
            ('LEFT', '左键', ""),
            ('RIGHT', '右键', "")
        ),
        default='LEFT'
    )

    ishelp: bpy.props.BoolProperty(
        name="显示帮助文本",
        description="Change the default state of the help text.",
        default=True
    )

    iscursorparam: bpy.props.BoolProperty(
        name="在游标处显示参数",
        description="Show the parameters next to the mouse cursor.",
        default=True
    )

    axis_bool: bpy.props.BoolProperty(
        name="显示轴的旋转",
        description="Show axis while moving mouse in view",
        default=True
    )

    object_ignorebehind: bpy.props.EnumProperty(
        name="过滤器",
        description="Hit only the grid or objects in front of it.",
        items=(
            ('ALL', "过滤全部", "", 0),
            ('FRONT', "仅网格前面的", "", 1),
            ('GRID', "仅网格", "", 2)
        ),
        default='ALL',
    )

    isignore_wire_bool: bpy.props.BoolProperty(
        name="忽略线框和边界框物体",
        description="If object viewport visibility is set to Wire or Bound, the ray will be recasted.",
        default=True
    )

    text_size_int: bpy.props.IntProperty(
        name="文本大小",
        description="Default segments count for cylinder and sphere.",
        default=14,
        min=4,
        max=128
    )

    header_color: bpy.props.FloatVectorProperty(
        name="标题颜色",
        description="The help section header color.",
        subtype='COLOR',
        default=[1.0, 0.38, 0.38, 1.0],
        size=4,
        min=0, max=1
    )

    text_color: bpy.props.FloatVectorProperty(
        name="文本颜色",
        description="The help section element color.",
        subtype='COLOR',
        default=[1.0, 1.0, 1.0, 1.0],
        size=4,
        min=0, max=1
    )

    hotkey_color: bpy.props.FloatVectorProperty(
        name="热键颜色",
        description="The help section hotkey color.",
        subtype='COLOR',
        default=[1.0, 0.86, 0.0, 1.0],
        size=4,
        min=0, max=1
    )

    # working plane settings #
    grid_count: bpy.props.IntProperty(
        name="网格格子数",
        description="The amount of grid division.",
        default=5,
        min=2,
        max=20
    )

    grid_color: bpy.props.FloatVectorProperty(
        name="网格线颜色",
        description="The color of the working plane grid lines.",
        subtype='COLOR',
        default=[0.655, 0.655, 0.655, 0.2],
        size=4,
        min=0, max=1
    )

    wpcursor_bool: bpy.props.BoolProperty(
        name="对齐3D游标",
        description="Align 3d cursor to working plane.",
        default=True
    )

    # Snap settings #
    zsnaptoggle_bool: bpy.props.BoolProperty(
        name="在“高度”阶段关闭捕捉。",
        description="Turn off snapping when in height set mode.",
        default=True
    )

    snap_dist: bpy.props.IntProperty(
        name="吸附距离",
        description="Snap distance between mouse position and snap points.",
        default=30,
        min=5,
        max=500
    )

    snap_dotsize: bpy.props.FloatProperty(
        name="点大小倍率",
        description="Snap dot size.",
        default=1.0,
        min=0.1,
        max=10
    )

    snap_closestColor: bpy.props.FloatVectorProperty(
        name="最近点颜色",
        description="The closest snap point color.",
        subtype='COLOR',
        default=[1.0, 1.0, 1.0, 1.0],
        size=4,
        min=0, max=1
    )

    snap_pointsColor: bpy.props.FloatVectorProperty(
        name="多边形点颜色",
        description="The snap points color.",
        subtype='COLOR',
        default=[1.0, 0.56, 0.0, 0.7],
        size=4,
        min=0, max=1
    )

    snap_gridpointsColor: bpy.props.FloatVectorProperty(
        name="网格点颜色",
        description="The snapgrid points color.",
        subtype='COLOR',
        default=[0.12, 0.56, 1.0, 0.7],
        size=4,
        min=0, max=1
    )

    # Object settings #
    segs_int: bpy.props.IntProperty(
        name="默认分段数",
        description="Default segments count for cylinder and sphere.",
        default=16,
        min=4,
        max=128
    )

    def draw(self, context):
        layout = self.layout
        # general settings
        general_box = layout.box()
        general_box.label(text="通用设置")
        general_box.label(text="插件的活动鼠标按钮:")
        general_box.row().prop(self, "mouse_enum", expand=True)
        general_box.row().prop(self, "ishelp")
        general_box.row().prop(self, "iscursorparam")
        general_box.row().prop(self, "axis_bool")
        general_box.row().prop(self, "text_size_int")
        general_box.row().prop(self, "header_color")
        general_box.row().prop(self, "text_color")
        general_box.row().prop(self, "hotkey_color")

        # raycast settings
        raycast_box = layout.box()
        raycast_box.label(text="光线投射设置")
        raycast_box.row().prop(self, "object_ignorebehind", expand=True)
        raycast_box.row().prop(self, "isignore_wire_bool")

        # wplane settings
        wplane_box = layout.box()
        wplane_box.label(text="工作平面设置")
        wplane_box.row().prop(self, "grid_count")
        wplane_box.row().prop(self, "grid_color")
        wplane_box.row().prop(self, "wpcursor_bool")

        # snap settings
        snap_box = layout.box()
        snap_box.label(text="捕捉设置")
        snap_box.row().prop(self, "zsnaptoggle_bool")
        snap_box.row().prop(self, "snap_dist")
        snap_box.row().prop(self, "snap_dotsize")
        snap_box.row().prop(self, "snap_closestColor")
        snap_box.row().prop(self, "snap_pointsColor")
        snap_box.row().prop(self, "snap_gridpointsColor")

        # object settings
        object_box = layout.box()
        object_box.label(text="物体设置")
        object_box.row().prop(self, "segs_int", expand=True)


# add menu object mode
class VIEW3D_MT_qbtools(bpy.types.Menu):
    bl_idname = "VIEW3D_MT_qbtools"
    bl_label = "QBlocker"

    def draw(self, context):
        layout = self.layout
        layout.operator("object.box_create", text="平面 / 立方体", icon='MESH_CUBE')
        layout.operator("object.cornercube_create", text="倒角立方体", icon='MESH_CUBE')
        layout.operator("object.cylinder_create", text="圆 / 圆柱", icon='MESH_CYLINDER')
        layout.operator("object.sphere_create", text="球体", icon='MESH_UVSPHERE')
        layout.operator("object.spherecube_create", text="四边面球体", icon='MESH_UVSPHERE')
        layout.operator("object.pyramid_create", text="金字塔", icon='MESH_UVSPHERE')
        layout.operator("object.torus_create", text="环体", icon='MESH_TORUS')
        layout.separator()
        layout.operator("object.convert_to_qblock", text="转换为 QBlock", icon='MESH_CUBE')
        # WIP
        layout.separator()
        layout.operator("object.qtransform", text="交互移动物体", icon='MESH_CUBE')


def qblocker_menu(self, context):
    self.layout.menu("VIEW3D_MT_qbtools")
    self.layout.separator()


# add menu edit mode
class VIEW3D_MT_qbtools_edit(bpy.types.Menu):
    bl_idname = "VIEW3D_MT_qbtools_edit"
    bl_label = "QBlocker"

    def draw(self, context):
        layout = self.layout
        layout.operator("object.box_create", text="P平面 / 立方体", icon='MESH_CUBE')
        layout.operator("object.cornercube_create", text="倒角立方体", icon='MESH_CUBE')
        layout.operator("object.cylinder_create", text="圆 / 圆柱", icon='MESH_CYLINDER')
        layout.operator("object.sphere_create", text="球体", icon='MESH_UVSPHERE')
        layout.operator("object.spherecube_create", text="四边面球体", icon='MESH_UVSPHERE')
        layout.operator("object.pyramid_create", text="金字塔", icon='MESH_UVSPHERE')
        layout.operator("object.torus_create", text="环体", icon='MESH_TORUS')
        # layout.separator()
        # layout.operator("object.convert_to_qblock", text="Convert to QBlock", icon='MESH_CUBE')


def qblocker_edit_menu(self, context):
    self.layout.menu("VIEW3D_MT_qbtools_edit")
    self.layout.separator()


classes = (
    BoxCreateOperator,
    CornerCubeCreateOperator,
    CylinderCreateOperator,
    SphereCreateOperator,
    SphereCubeCreateOperator,
    PyramidCreateOperator,
    TorusCreateOperator,
    ConvertToQBlock,
    ObjectTransformOperator,
    VIEW3D_MT_qbtools,
    VIEW3D_MT_qbtools_edit,
    QBlockProperties,
    MESH_PT_QblockPanel
)


def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    # bpy.types.VIEW3D_MT_object_context_menu.prepend(qblocker_menu)
    bpy.types.VIEW3D_MT_add.prepend(qblocker_menu)
    bpy.types.VIEW3D_MT_mesh_add.prepend(qblocker_edit_menu)
    bpy.utils.register_class(QBlockerAddonPreferences)
    bpy.types.Mesh.qblock_props = bpy.props.PointerProperty(type=QBlockProperties)


def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    # bpy.types.VIEW3D_MT_object_context_menu.remove(qblocker_menu)
    bpy.types.VIEW3D_MT_add.remove(qblocker_menu)
    bpy.types.VIEW3D_MT_mesh_add.remove(qblocker_edit_menu)
    bpy.utils.unregister_class(QBlockerAddonPreferences)


if __name__ == "__main__":
    register()
